/*
  Copyright 1996-2000
  Simon Whiteside

* $Id: Executable.java,v 1.10 2001/01/02 11:51:24 sdw Exp $
*/

package simkin;

/**
 * This interface is implemented by objects which want to override the method and field access 
 * which uses Java introspection by default.
 * <p>An executable object is considered by the interpreter to consists of properties and
methods. The interface defines 3 major functions: <ul>
 * <li>{@link simkin.Executable#getValue getValue} : gets a value from the object
 * <li>{@link simkin.Executable#setValue setValue} : sets a value in the object
 * <li>{@link simkin.Executable#method method} : calls a method on the object
 * </ul>
 */
public interface Executable {
    /**
     * Called to set the value of a field. If an attribute name is given, the value is stored as an attribute of the given element, otherwise as a text value
     * @param field_name the name of the field
     * @param attrib_name the name of the attribute to be set (can be null)
     * @param value the value to be set
     * @throws RuntimeException - if there was a problem running the script (such as not having permission to access a field)
     * @throws NoSuchFieldException - if the field could not be found
     */
    public void setValue(String field_name,String attrib_name,Object value) throws RuntimeException, NoSuchFieldException;
    /**
     * Called to get the value of a field
     * @param field_name the name of the field
     * @param attrib_name attribute name (null if no attribute specified)
     * @return the value of the field (or null, if the field is not supported)
     * @throws RuntimeException - if there was a problem running the script (such as not having permission to access a field)
     * @throws NoSuchFieldException - if the field could not be found
     */
    public Object getValue(String field_name,String attrib_name) throws RuntimeException, NoSuchFieldException;
    /**
     * Called if a Simkin script calls a method on this object. 
     * @param method_name the name of the method
     * @param arguments an array of arguments passed to the method
     * @return the function's return value
     * @throws RuntimeException - if there was a problem running the script (such as divide by zero)
     * @throws NoSuchMethodException - if the method could not be found
     */
    public Object method(String method_name,Object[] arguments) throws ParseException, RuntimeException, NoSuchMethodException;
};
